#include <vector>
#include <iostream>
#include <sstream>
#include <fstream>
#include <cmath>
using namespace std;

typedef vector<double> row;

class CperformanceEvaluator
{
public:
	CperformanceEvaluator(){};
	~CperformanceEvaluator(){};

	// Has to read the contents of the file
	void getInputDir(const char *filename) {m_inputDir = filename;}
	void readFile();
	// Or receive the coordinates directly and generate m_matrix from it
	void getM_matrix(vector<vector<double>> coordinates);
	// Has to calculate the performance
	void calcPerformance();
	// Write cost in the folder
	void writeCost();
	void getOutputDir(const char *filename) {m_outputDir = filename;}
	// Let's just print out m_matrx
	void printM_matrix();
	// To acces m_cost
	double getCost(){return m_cost;}


	vector<row> m_matrix;
	double m_cost;
	const char *m_inputDir;
	const char *m_outputDir;
	#define SCALING 1000
	#define DESIRED_DISTANCE_FB 50 / SCALING
	#define DESIRED_DISTANCE_LR 50 / SCALING
	#define DESIRED_DISTANCE_DIAG 70 / SCALING

	
};


void getM_matrix(vector<vector<double>> coordinates){
	vector<double> angle_array;

	// Calculate the angle

	// Compare it with the previous angle and if necessarely, change it

	// Combine the coordinates & the angle_array and make it equal with m_matrix
}

void CperformanceEvaluator::writeCost(){
	ofstream file(m_outputDir);
	file << m_cost;
	file.close();
	clog << "The error is: " << m_cost << endl;

}

void CperformanceEvaluator::calcPerformance(){
	m_cost = 0.0;
	for (int i = 0; i < m_matrix.size(); ++i)
	{
		if(m_matrix[i].size() == 0) {break;}
		vector<double> tmp_row = m_matrix[i];

		// Coordinates
		double O2x = tmp_row[3];
		double O2y = tmp_row[4];
		double O4x = tmp_row[5];
		double O4y = tmp_row[6];
		double O6x = tmp_row[7];
		double O6y = tmp_row[8];
		double true_angle = tmp_row[9];

		// Distances
		double dist_fbK4   = sqrt( pow(O2x - O4x,2) + pow(O2y - O4y, 2) );
	    double dist_lrK4   = sqrt( pow(O4x - O6x,2) + pow(O4y - O6y, 2) );
		double dist_diagK4 = dist_fbK4;
        double dist_fbK6   = sqrt( pow(O2x - O6x,2) + pow(O2y - O6y, 2) );

        // Diversions
        double div_fbK4   = abs(dist_fbK4 - DESIRED_DISTANCE_FB);
        double div_lrK4   = abs(dist_lrK4 - DESIRED_DISTANCE_LR);
        double div_diagK4 = abs(dist_diagK4 - DESIRED_DISTANCE_DIAG);
        double div_fbK6   = abs(dist_fbK6 - DESIRED_DISTANCE_FB);
        double div_angle  = abs(true_angle);

        // Cost
        m_cost = m_cost + div_fbK4 + div_lrK4 + div_diagK4 + div_fbK6 + div_angle;
	}
}

void CperformanceEvaluator::printM_matrix(){
		for (int i = 0; i < m_matrix.size(); ++i)
		{
			for (int j = 0; j < m_matrix[i].size(); ++j)
			{
				clog << m_matrix[i][j] << ",";
			}
			clog << endl;
		}
	}

void CperformanceEvaluator::readFile() {
		ifstream file ( m_inputDir ); // declare file stream: http://www.cplusplus.com/reference/iostream/ifstream/
		string line_s;
		while ( file.good() )
		{
	
		    getline ( file, line_s, '\n' ); // read a string until next comma: http://www.cplusplus.com/reference/string/getline/
		    string value_s;
			stringstream ss(line_s);
			vector<double> tmp_row;
			while(std::getline(ss, value_s, ',')) {
	    		//std::cout << value << '\n';
	    		double tmp_value_d = stod(value_s);
	    		//cout << tmp_value_d << endl;
	    		tmp_row.push_back(tmp_value_d);
			}
			m_matrix.push_back(tmp_row);
		}
		file.close();
	
}

/*
int main(int argc, char const *argv[])
{
	CperformanceEvaluator cevaluator;
	const char filename[] = "/Users/szilard/argos3-kilobot-master/src/examples/behaviors/data/Trial_0/coords4.csv";
	cevaluator.readFile(filename);
	cevaluator.calcPerformance();
	const char to_write[] = "/Users/szilard/argos3-kilobot-master/src/examples/behaviors/data/Trial_0/cost.csv";
	cevaluator.writeCost(to_write);

	clog << cevaluator.getCost() << endl;
	return 0;
}
*/