#include <kilolib.h>
#include "test_debug.h"
#include <debug.h>
//#include "extra_fn.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

// Constants for orbit control.
#define TOO_CLOSE_DISTANCE 30
#define DESIRED_DISTANCE_FB 50
#define DESIRED_DISTANCE_LR 50
#define DESIRED_DISTANCE_DIAG 70
#define TRANSVERSAL_CONSTANT 0

// Constants for motion handling function.
#define STOP 0
#define FORWARD 1
#define LEFT 2
#define RIGHT 3

// Constant for handling multiple stars
#define MEMORY_SIZE 4      //if changed, change function "lets_move" & "get_smallest_distance" as well --> defines the number of agents stored
#define NUM_OF_ROWS 3      //if changed, change function "lets_move" & "get_smallest_distance" as well --> defines the number of "features" stored
#define ID_ROW 0           
#define DISTANCE_ROW 1
#define UPDATED_ROW 2
#define REQUIRED_NUMBER 1  //defines how many agents have to be in "hearing-range"

// Constants for easier handling of FRONT, BACK, LEFT, RIHT
#define FRONT 0
#define BACK 1
#define LEFT 2
#define RIHT 3
#define FRONTLEFT 4
#define FRONTRIGHT 5
#define BACKLEFT 6
#define BACKRIGHT 7

// Constants for easier handling of NEW, OLD
#define NEW 0
#define OLD 1

// Constants for easier handling of LONGITUDINAL, TRANSVERSAL
#define LONGITUDINAL 0
#define TRANSVERSAL 1

#define DISTANCE 0
#define TICK 1

#define FILTER_WINDOW 100
#define PI 3.14159265358979323846

#define FIRST_ROW_LONGITUDINAL_SPEED 400
#define FIRST_ROW_TRANSVERSAL_SPEED 50

#define WAIT_TIME 40

double arr[FILTER_WINDOW] = {0};
double arr_filtered = 0;
int add_this_sign = 1; //1 if positive, 0 if negative
int add_this_sign_changed = 0;

message_t message;
// Stuff needed for multiple stars
int received_ID = 0;
//int smallest_dist = 255;
int listen_for_message = 1;


int current_motion = STOP;
int distance;
int new_message = 0;
double current_time;
double current_time_old;
double elapsed_time;

//double diversion_old;
//double diversion_integral_old;
//double diversion_derivative_old;
double delay_pid_longitudinal;
double delay_pid_transversal;
double delay_pid_longitudinal_old = 0;
double delay_pid_transversal_old = 0;
double delay_pid_transversal_history[FILTER_WINDOW] = {0};
double true_angle = 0;
double delay_angle = 0;
double cp_longitudinal;
double ci_longitudinal;
double cd_longitudinal;
double cp_transversal_lr;
double ci_transversal_lr;
double cd_transversal_lr;
double cp_transversal_diag;
double ci_transversal_diag;
double cd_transversal_diag;
double cp_angle;
int distances             [BACKRIGHT + 1][OLD + 1];
int diversion             [BACKRIGHT + 1][OLD + 1];
double diversion_integral    [BACKRIGHT + 1][OLD + 1];
double diversion_derivative  [BACKRIGHT + 1][OLD + 1] = {{0}};
int neighbours            [BACKRIGHT + 1];

double proportional_part_longitudinal;
double integral_part_longitudinal;
double derivative_part_longitudinal;
double proportional_part_transversal;
double integral_part_transversal;
double derivative_part_transversal;

double ML_angle = 0;
double angle;
double angle_old;
double angle1;
double angle1_old;
double angle2;
double angle2_old;

uint32_t prev_time = 0;
double sync_delay = 0;



uint32_t step_count = 0;
typedef int bool;
#define false 0
#define true 1
bool my_step = false;

//"Circle" variables
double O2x, O2x_prev, O2y, O2y_prev;
double O4x, O4y;
double O6x, O6y;

double r24, r24_prev;
double r26, r26_prev;


// ----- Structs for wrapping data ----- //
struct SGPS_data{
  double x;
  double y;
  double z;
  double rotZ;
};

#define MULTIPLIER_INPUT 2
#define MULTIPLIER_NEURON 3
#define M1_DIM1 20 * MULTIPLIER_NEURON
#define M1_DIM2 9 * MULTIPLIER_INPUT
#define M2_DIM1 9 * MULTIPLIER_INPUT
#define M2_DIM2 1
#define X_LENGTH 9

double x_hist[M1_DIM2];
double x_hist_tmp[M1_DIM2];
double x[M1_DIM2];

struct Scoeff_matrices
{
  // Step1
  double offset1[M1_DIM2];
  double gain1[M1_DIM2];
  double min1[M1_DIM2];
  // Step2
  double gain2[M1_DIM1][M1_DIM2];
  double bias22[M1_DIM1];
  //Step3
  double gain3[M1_DIM1];
  double bias3;
  //Step4
  double min4;
  double gain4;
  double offset4;
  
};

struct Seffectivenes_data
{
  double one;
  double two;
  double three;
  double four;
  double five;
  double six;
  double seven;
  double eight;
};

struct Spid_values
{
  double cp_longitudinal;
  double ci_longitudinal;
  double cd_longitudinal;
  double cp_transversal_lr;
  double ci_transversal_lr;
  double cd_transversal_lr;
  double cp_transversal_diag;
  double ci_transversal_diag;
  double cd_transversal_diag;
  double cp_angle;
};

// Struct declarations
struct Scoeff_matrices coeff_matrices;
struct Seffectivenes_data effectivenes_data;
struct Seffectivenes_data effectivenes_data2;
struct Seffectivenes_data effectivenes_data4;
struct Seffectivenes_data effectivenes_data6;
struct Seffectivenes_data effectivenes_data_received;

struct SGPS_data GPS_data_2;
struct SGPS_data GPS_data_4;
struct SGPS_data GPS_data_6;
struct SGPS_data vector_with_angle;
struct SGPS_data vector_with_angle_2;


// ------ Function declarations ------ //
// Function to handle motion.
void set_motion(int new_motion);
//PID functions
void calculate_pid_delay_longitudinal();
void calculate_pid_delay_transversal();
//Miscellaneous
void calculate_effectiveness(double current_time);
struct SGPS_data get_location();
void write_data_MATLAB(double current_time);
//Messages functions
message_t *message_tx();
void message_rx(message_t *m, distance_measurement_t *d);
//ML - For estimating the angle
void array_manipulate(double *res, char *command, double *to_manip, int to_manip_size, double *manip_with, int manip_with_size);
void matrix_multiply(double (*res_matrix)[M2_DIM2], double (*matrix1)[M1_DIM2], int m1_dim1, int m1_dim2, double (*matrix2)[M2_DIM2], int m2_dim1, int m2_dim2);
//void print_matrix(double (*matrix)[M1_DIM2], int m1_dim1, int m1_dim2);
struct Scoeff_matrices read_ML_coeff();
void print_Scoeff_matrices(struct Scoeff_matrices coeff_matrices);
//struct Seffectivenes_data get_effectivenes(int id_number);
double angle_calculation(double x1[M1_DIM2]);

void read_PID_values();
void read_neighbours();
void performance_eval();

void write_coords();


//------ Function declarations END ------//


void setup() {

    /*
    //To zero-out .csv file holding delay data. Containst calculate_effectivenes() data.
    //More(-or less) precisely: (double)kilo_ticks, one, two, three, four, five, six, seven, eight
    FILE *fPointer;
    char str[100];
    sprintf(str, "src/examples/behaviors/data/data%i.csv", kilo_uid);
    fPointer = fopen(str, "w");
    fclose(fPointer);

    //To zero-out .csv file holding GOD Mode data for Matlab. Containst write_data_MATLAB() data.
    //More(-or less) precisely:
    FILE *fPointer_GOD;
    char str_GOD[100];
    sprintf(str_GOD, "src/examples/behaviors/data/data_GOD%i.csv", kilo_uid);
    fPointer_GOD = fopen(str_GOD, "w");
    fclose(fPointer_GOD);

    //ML stuff - we are just clearing the cvs files
    FILE *fPointer2;
    char str2[100];
    sprintf(str2, "src/examples/behaviors/data/data%i_2.csv", kilo_uid);
    fPointer2 = fopen(str2, "w");
    fclose(fPointer2);


    FILE *fPointer4;
    char str4[100];
    sprintf(str4, "src/examples/behaviors/data/data%i_4.csv", kilo_uid);
    fPointer4 = fopen(str4, "w");
    fclose(fPointer4);


    FILE *fPointer6;
    char str6[100];
    sprintf(str6, "src/examples/behaviors/data/data%i_6.csv", kilo_uid);
    fPointer6 = fopen(str6, "w");
    fclose(fPointer6);
    //ML stuff - we are just clearing the cvs files END

    
    {
    FILE *fPointer_GOD;
    char str_GOD[100];
    sprintf(str_GOD, "src/examples/behaviors/data/coords%i.csv", kilo_uid);
    fPointer_GOD = fopen(str_GOD, "w");
    fclose(fPointer_GOD);
    }
    

    */
    coeff_matrices = read_ML_coeff();
    if(kilo_uid == 4) {
      //print_Scoeff_matrices(coeff_matrices);
    }

    //if (kilo_uid == 4)
    //{
    //  print_Scoeff_matrices(coeff_matrices);
    //}

    //Initializing message
    message.type = NORMAL;
    message.data[0] = kilo_uid;
    message.crc = message_crc(&message);

    read_PID_values();
    //Reading PID values
    /*
    double *pid;
    pid = read_PID_values();
    cp_longitudinal = *pid;
    ci_longitudinal = *(pid + 1);
    cd_longitudinal = *(pid + 2);
    cp_transversal_lr  = *(pid + 3);
    ci_transversal_lr  = *(pid + 4);
    cd_transversal_lr  = *(pid + 5);
    cp_transversal_diag  = *(pid + 6);
    ci_transversal_diag  = *(pid + 7);
    cd_transversal_diag  = *(pid + 8);
    cp_angle             = *(pid + 9);
    */

    read_neighbours();
    
    current_time = kilo_ticks;    
    
    //Initialize diversions
    delay(1000);
    //The front-back
    for (int i = FRONT; i < BACK + 1; i++) {
      diversion[i][NEW] = distances[i][NEW] - DESIRED_DISTANCE_FB; //Positive, if too far 
      diversion[i][OLD] = diversion[i][NEW];   
    }
    //The left-right
    for (int i = LEFT; i < RIGHT + 1; i++) {
      diversion[i][NEW] = distances[i][NEW] - DESIRED_DISTANCE_LR; //Positive, if too far 
      diversion[i][OLD] = diversion[i][NEW];   
    }
    //The diagonals
    for (int i = FRONTLEFT; i < BACKRIGHT + 1; i++) {
      diversion[i][NEW] = distances[i][NEW] - DESIRED_DISTANCE_DIAG; //Positive, if too far
      diversion[i][OLD] = diversion[i][NEW];       
    }
    delay(1000);
    current_time_old = current_time;

    
    for (int i = 0; i < M1_DIM2; ++i)
    {
        x_hist_tmp[i] = 0;
        x_hist[i] = 0;
    }

    GPS_data_2 = get_location(2);
    GPS_data_4 = get_location(4);
    GPS_data_6 = get_location(6);

}




      
      

        





//------------------------------------ Loop ------------------------------------//
void loop() {
  //debug_print("step_count = %i\n", step_count);


  // ---- Things related to angle ---- //
            GPS_data_2 = get_location(2);
            GPS_data_4 = get_location(4);
            GPS_data_6 = get_location(6);
            //First vector
            vector_with_angle.x = GPS_data_4.x - GPS_data_6.x;
            vector_with_angle.y = GPS_data_4.y - GPS_data_6.y;
            vector_with_angle.z = GPS_data_4.z - GPS_data_6.z;
            //Second vector
            vector_with_angle_2.x = GPS_data_2.x;
            vector_with_angle_2.y = GPS_data_2.y;
            vector_with_angle_2.z = GPS_data_2.z;

            //Calculating angle with cosine
            //double a_times_b = vector_with_angle.x * vector_with_angle_2.x + vector_with_angle.y * vector_with_angle_2.y + vector_with_angle.z * vector_with_angle_2.z;
            //double a_abs = sqrt(  pow(vector_with_angle.x, 2) + pow(vector_with_angle.y, 2) + pow(vector_with_angle.z, 2) );
            //double alfa = 
            //Calculating angle
                                                                                      //https://stackoverflow.com/questions/14066933/direct-way-of-computing-clockwise-angle-between-2-vectors  
            double dot = 1.0 * vector_with_angle.x + 0.0 * vector_with_angle.y;       // dot product between [x1, y1] and [x2, y2]
            double det = 1.0 * vector_with_angle.y - 0.0 * vector_with_angle.x;       // determinant
            
            angle = atan2(det, dot);  // atan2(y, x) or atan2(sin, cos)
            vector_with_angle.rotZ = angle;
            angle1 = vector_with_angle.rotZ/ PI * 180.0 - 90;
            angle2 = GPS_data_2.rotZ / PI * 180;

            ML_angle = ML_angle > 40 ? 40 : ML_angle;
            true_angle = angle2 - angle1;
            //if (kilo_uid == 4)
            //{
            //  debug_print("kilo_ticks = %u, true_angle = %f, angle1 = %f, angle2 = %f, angle1_old = %f, angle2_old = %f\n", kilo_ticks, true_angle, angle1, angle2, angle1_old, angle2_old);
            //}
            angle1 = angle1 > angle1_old + 10 ? angle1-360 :  angle1;
            angle1 = angle1 < angle1_old - 10 ? angle1+360 :  angle1;

            angle2 = angle2 > angle2_old + 10 ? angle2-360 :  angle2;
            angle2 = angle2 < angle2_old - 10 ? angle2+360 :  angle2;
            true_angle = angle2 - angle1;
            //if (kilo_uid == 4)
            //{
            //  debug_print("kilo_ticks = %u, true_angle = %f, angle1 = %f, angle2 = %f, angle1_old = %f, angle2_old = %f\n", kilo_ticks, true_angle, angle1, angle2, angle1_old, angle2_old);
            //}
            angle_old = angle;
            angle1_old = angle1;
            angle2_old = angle2;


            delay_angle = cp_angle * (angle2 - angle1);




          // ---- Deciding who's step ---- //
          if (kilo_uid == 4)
          {
            //printf("K2(%f, %f),K4(%f, %f),K6(%f, %f)\n", GPS_data_2.x, GPS_data_2.y, GPS_data_4.x, GPS_data_4.y, GPS_data_6.x, GPS_data_6.y );
          }
          my_step = false;
          if ( !(step_count % 2)  ) {
            if (kilo_uid == 2)
            {
              my_step = true;
            }
            
            // Circle calculations
            O2x_prev = GPS_data_2.x;
            O2y_prev = GPS_data_2.y;
          }
          if ( (step_count % 2) && ((kilo_uid == 4) || (kilo_uid == 6)) ) {
            my_step = true;
            // CIrcle calculations
            O2x = GPS_data_2.x;
            O2y = GPS_data_2.y;

            O4x = GPS_data_4.x;
            O4y = GPS_data_4.y;

            O6x = GPS_data_6.x;
            O6y = GPS_data_6.y;

            // Write coords
            //write_coords();

            //if (kilo_uid == 4)
            //{
            //  debug_print("O2_prev(%f, %f) & O2(%f, %f) | O4(%f, %f) | O6(%f, %f)  --  r24, r24_prev : (%f, %f), r26, r26_prev : (%f, %f)\n", 
            //  O2x_prev, O2y_prev, O2x, O2y, O4x, O4y, O6x, O6y, r24, r24_prev, r26, r26_prev );
            //}


            // OK. We have the previous data & now the new data
            r24 = sqrt( pow(O2x - O4x,2) + pow(O2y - O4y, 2) );
            r26 = sqrt( pow(O2x - O6x,2) + pow(O2y - O6y, 2) );

            r24_prev = sqrt( pow(O2x_prev - O4x,2) + pow(O2y_prev - O4y, 2) );
            r26_prev = sqrt( pow(O2x_prev - O6x,2) + pow(O2y_prev - O6y, 2) );


            //Moove the coordinate system:
            O2x = O2x - O2x_prev;
            O2y = O2y - O2y_prev;

            // Let's calculate things
            double a = pow(r24, 2) - pow(r24_prev, 2) - pow(O2x, 2) - pow(O2y, 2);
            double b = 2.0 * O2y;
            double c = - 2.0 * O2x;

            double c1 = 1.0 + pow(b, 2) / pow(c, 2);
            double c2 = 2.0 * a * b / pow(c, 2);
            double c3 = pow(a, 2) / pow(c, 2) - pow(r24_prev,2);

            double ans1 = - c2 - sqrt(  pow(c2,2) - 4.0 * c1 * c3  );
            ans1 = ans1 / (2.0 * c1);
            
            double ans2 = sqrt(  pow(r24_prev,2) - pow(ans1,2)  );
            double ans2_2 = sqrt(  pow(r24_prev,2) + pow(ans1,2)  );

            // Moove back the coordinate system
            ans1 = ans1 + O2y_prev;
            ans2 = ans2 + O2x_prev;
            ans2_2 = ans2_2 + O2x_prev;

            //if (kilo_uid == 4)
            //{
            //  printf("\na = %f, b = %f, c = %f\n c1 = %f, c2 = %f, c3 = %f\n", a, b, c, c1, c2, c3 );
            //  printf("--> O4(%f, %f) <-> Calc. O4(%f or %f, %f)\n\n", O4x, O4y, ans2, ans2_2, ans1 );
            //}

            //Moove the coordinate system back:

          }


            // ---- Things related to angle ---- //
            
                x[0]  = effectivenes_data2.eight;

                x[1]  = effectivenes_data4.one;
                      //effectivenes_data4.two;
                x[2]  = effectivenes_data4.three;
                      //effectivenes_data4.four;
                      //effectivenes_data4.five;
                      //effectivenes_data4.six;
                x[3]  = effectivenes_data4.seven;
                x[4]  = effectivenes_data4.eight;

                x[5]  = effectivenes_data6.one;
                x[6]  = effectivenes_data6.two;
                      //effectivenes_data6.three;
                      //effectivenes_data6.four;
                      //effectivenes_data6.five;
                      //effectivenes_data6.six;
                x[7] = effectivenes_data6.seven;
                x[8] = effectivenes_data6.eight;

                
              //Update x_hist
              if (my_step) {
                for (int i = 0; i < M1_DIM2; ++i)
                {
                    x_hist_tmp[i] =  x_hist[i];
                }

                for (int i = 0; i < M1_DIM2 - X_LENGTH; ++i)
                {
                  x_hist[i] = x_hist_tmp[i + X_LENGTH];
                }

                for (int i = M1_DIM2 - X_LENGTH; i < M1_DIM2; ++i)
                {
                  x_hist[i] = x[i - (M1_DIM2 - X_LENGTH) ];
                }
              }


          //------ P ------//
          //The front-back
          for (int i = FRONT; i < BACK + 1; i++) {
            diversion[i][NEW] = distances[i][NEW] - DESIRED_DISTANCE_FB;    //Positive, if too far    
          }
          //The left-right
          for (int i = LEFT; i < RIGHT + 1; i++) {
            diversion[i][NEW] = distances[i][NEW] - DESIRED_DISTANCE_LR;    //Positive, if too far    
          }
          //The diagonals
          for (int i = FRONTLEFT; i < BACKRIGHT + 1; i++) {
            diversion[i][NEW] = distances[i][NEW] - DESIRED_DISTANCE_DIAG;  //Positive, if too far    
          }

          //------ Zero out------//     
          for (int i = 0; i < BACKRIGHT + 1; i++) {                         //if given neighbour is not considered
            if(neighbours[i] == 0) {
              diversion[i][NEW] = 0;
              diversion[i][OLD] = 0;
              diversion_integral[i][NEW] = 0;
              diversion_derivative[i][NEW] = 0;
            }
          }

          //------ Time management------// 
          current_time = kilo_ticks;
          elapsed_time = current_time - current_time_old;
          elapsed_time = 100;
          //debug_print("elapsed_time = %f\n", elapsed_time);
          if (elapsed_time == 0) {
            debug_print("This message should not show: The elapsed_time was 0. That is not good.\n");
            elapsed_time = 1;
          }                        //To protect from dividing by zero, just for good measure.

          //------ I ------//
          for (int i = 0; i < BACKRIGHT + 1; i++) {
            diversion_integral[i][NEW] = diversion_integral[i][NEW] + (elapsed_time) * diversion[i][NEW]; //Positive, if too far            
          }

          //------ D ------//                                              //(with a bit of noise reduction)
          for (int i = 0; i < BACKRIGHT + 1; i++) {
            diversion_derivative[i][NEW] = (  5.0 * diversion_derivative[i][NEW] + ((double)diversion[i][NEW] - (double)diversion[i][OLD]) / elapsed_time   ) / 6.0; //positive if we are even further from the desired --> positive if we are distancing ourselves with a faster rate 
          }
          
          

      if (my_step) {
          //------ Longitudinal Delay ------//
          calculate_pid_delay_longitudinal();
           
          //------ Transversal Delay ------//
          calculate_pid_delay_transversal();
      } else {
            delay_pid_longitudinal = 0;
            delay_pid_transversal = 0;
      }
          //------ Motion ------//
          //-- First row only --//
          if (kilo_uid == 1 || kilo_uid == 2 || kilo_uid == 3) {
            if(my_step){ 
              set_color(RGB(1, 0, 0));
              //debug_print("step_count = %i, I am moving!\n", step_count);
             }
            
            
            delay_pid_longitudinal = my_step == 1 ? FIRST_ROW_LONGITUDINAL_SPEED : 0; //azért, hogy a Kilobot 2-nek 0 legyen az értéke, hogy a ML tudjon dolgozni
              set_motion(FORWARD);
              delay(delay_pid_longitudinal);
              set_motion(STOP);

            // Add a bit of randomnes, if we are the first row
            if ((rand_hard()%2) && (rand_hard()%2) && (rand_hard()%2)) {
              delay_pid_transversal = (double)(rand_hard()%2 + 2); //azért, hogy a Kilobot 2-nek 0 legyen az értéke, hogy a ML tudjon dolgozni
                                                          //ebből nyilván csak arra jövünk rá, hogy melyik irányba fordul, de a ML majd magától
                                                          //rájön, hogy hogyan kezelje ezt az adatot.
                                                          //Mondjuk érdemes lehet megnézni, hogy miért 127, 129, 130 értéket vesz föl és ez nem baj-e
              set_motion(delay_pid_transversal);
              delay_pid_transversal = my_step == 1 ? FIRST_ROW_TRANSVERSAL_SPEED : 0;
              delay(delay_pid_transversal);
              set_motion(STOP);
            }
            

          }
          //Then they all wait a bit
          
          //set_color(RGB(1, 1, 0));
          sync_delay = WAIT_TIME + 32 - (kilo_ticks - prev_time);
          sync_delay = sync_delay / 32.0;
          sync_delay = sync_delay * 1000.0;
          delay( (int)sync_delay );
          prev_time = kilo_ticks;
          set_color(RGB(0, 0, 0));
          //debug_print("I waited %f ms after first row moove\n", sync_delay);
          //-- Everybody else --//
      if(my_step) {
          if(kilo_uid != 1 && kilo_uid != 2 && kilo_uid != 3) {
            set_color(RGB(0, 1, 0));
            //debug_print("step_count = %i, I am moving!\n", step_count);
            
            
            //PID motion
            //Only move forward, if value > 0. Else we should go backwards bat that is not possible
                                              //The first row cannot be forced to moove quicker. Only to slow down.
            if ( delay_pid_longitudinal > 0 ) {
              set_motion(FORWARD);
              delay((int)delay_pid_longitudinal);
              set_motion(STOP);
            } else {
              delay_pid_longitudinal = 0; //Ez fontos!! Ha nem kezdünk el hátrálni, akkor ne is hagyjuk meg negatívnak az adatot, mert későbbi számításoknál baj lehet
            }
          


          
            //Everybody obeys the transversal forces
            delay_pid_transversal = delay_pid_transversal - delay_angle;
            if ( (delay_pid_transversal > 0)  && (diversion[FRONTLEFT][NEW] > -6) && (diversion[FRONT][NEW] > -6) && (diversion[FRONTRIGHT][NEW] > -6)  ) {
              set_motion(LEFT);
              delay((int)(  delay_pid_transversal + TRANSVERSAL_CONSTANT) );
              set_motion(STOP);
            } else if (delay_pid_transversal < 0  && (diversion[FRONTLEFT][NEW] > -6) && (diversion[FRONT][NEW] > -6) && (diversion[FRONTRIGHT][NEW] > -6)  ) {
              set_motion(RIGHT);
              delay(  abs((int)(delay_pid_transversal))  + TRANSVERSAL_CONSTANT );
              set_motion(STOP);
            } else {
              set_motion(STOP);
              delay(  abs((int)delay_pid_transversal) );
              delay_pid_transversal = 0;
              //debug_print("This message should not show: I am not doing anything, because I am too close!\n");
            }
  

          }
      }


          //Waiting for synching: WAIT FOR MOVEMENTS TO FINISH
          //32 ticks per second. Approximately 470ms per 15 ticks. We wait about 340ms after moving to synch up our messages.
          set_color(RGB(0, 0, 0));
          
          //set_color(RGB(1, 1, 0));
          sync_delay = WAIT_TIME - (kilo_ticks - prev_time);
          sync_delay = sync_delay / 32.0;
          sync_delay = sync_delay * 1000.0;
          delay( (int)sync_delay );
          //debug_print("I waited %f ms after second row moove\n", sync_delay);
          prev_time = kilo_ticks;
          set_color(RGB(0, 0, 0));
          
          

          //------ Writing data ------//
          //r24 = distances [FRONT][DISTANCE];
          //r26 = effectivenes_data6.one;
          
          //calculate_effectiveness((double)current_time);
          //write_data_MATLAB((double)current_time);
          

          //------ Update ------//
          for (int i = 0; i < BACKRIGHT + 1; i++) {
            diversion[i][OLD] = diversion[i][NEW];
          }
          current_time_old = current_time;

          step_count = step_count + 1;

} //END of loop



int main()
{

   kilo_init();
   kilo_message_tx = message_tx;
   kilo_message_rx = message_rx;
   debug_info_create();
   kilo_start(setup, loop);
   return 0;
}

//------------------------------------------ END OF CODE ------------------------------------------//





























void write_coords(){
    FILE *fPointer_GOD;
    char str_GOD[100];
    sprintf(str_GOD, "src/examples/behaviors/data/coords%i.csv", kilo_uid);
    fPointer_GOD = fopen(str_GOD, "a");
    fprintf(fPointer_GOD, "%u, %f ,%f, %f, %f, %f, %f, %f, %f, %f\n", 
      kilo_ticks, O2x_prev, O2y_prev, O2x, O2y, O4x, O4y, O6x, O6y, true_angle);
    //printf(fPointer_GOD, "kappa\n");
    fclose(fPointer_GOD);
}

void performance_eval(){
  
}



void read_neighbours(){
    FILE *fPointer;
    char str[100];
    sprintf(str, "src/examples/behaviors/neighbours/neighbours%i.csv",kilo_uid);
    fPointer = fopen(str, "r");

    char neighbours_front_s[10];
    char neighbours_back_s[10];
    char neighbours_left_s[10];
    char neighbours_right_s[10];
    char neighbours_frontleft_s[10];
    char neighbours_frontright_s[10];
    char neighbours_backleft_s[10];
    char neighbours_backright_s[10];

    fscanf(fPointer, "%s", neighbours_front_s);
    fscanf(fPointer, "%s", neighbours_back_s);
    fscanf(fPointer, "%s", neighbours_left_s);
    fscanf(fPointer, "%s", neighbours_right_s);

    fscanf(fPointer, "%s", neighbours_frontleft_s);
    fscanf(fPointer, "%s", neighbours_frontright_s);
    fscanf(fPointer, "%s", neighbours_backleft_s);
    fscanf(fPointer, "%s", neighbours_backright_s);

    char *p;
    
    neighbours[FRONT]       = strtof(neighbours_front_s, &p);
    neighbours[BACK]        = strtof(neighbours_back_s, &p);
    neighbours[LEFT]        = strtof(neighbours_left_s, &p);
    neighbours[RIGHT]       = strtof(neighbours_right_s, &p);
    neighbours[FRONTLEFT]   = strtof(neighbours_frontleft_s, &p);
    neighbours[FRONTRIGHT]  = strtof(neighbours_frontright_s, &p);
    neighbours[BACKLEFT]    = strtof(neighbours_backleft_s, &p);
    neighbours[BACKRIGHT]   = strtof(neighbours_backright_s, &p);
}

void read_PID_values(){
    FILE *fPointer;
    char str[100];
    sprintf(str, "src/examples/behaviors/pid_values.csv", kilo_uid);
    fPointer = fopen(str, "r");

    //Step1
    char cp_longitudinal_s[10];
    char ci_longitudinal_s[10];
    char cd_longitudinal_s[10];
    char cp_transversal_lr_s[10];
    char ci_transversal_lr_s[10];
    char cd_transversal_lr_s[10];
    char cp_transversal_diag_s[10];
    char ci_transversal_diag_s[10];
    char cd_transversal_diag_s[10];
    char cp_angle_s[10];

    fscanf(fPointer, "%s", cp_longitudinal_s);
    fscanf(fPointer, "%s", ci_longitudinal_s);
    fscanf(fPointer, "%s", cd_longitudinal_s);

    fscanf(fPointer, "%s", cp_transversal_lr_s);
    fscanf(fPointer, "%s", ci_transversal_lr_s);
    fscanf(fPointer, "%s", cd_transversal_lr_s);

    fscanf(fPointer, "%s", cp_transversal_diag_s);
    fscanf(fPointer, "%s", ci_transversal_diag_s);
    fscanf(fPointer, "%s", cd_transversal_diag_s);

    fscanf(fPointer, "%s", cp_angle_s);

    //
    char *p;

    //Reading
    cp_longitudinal = strtof(cp_longitudinal_s, &p);
    ci_longitudinal = strtof(ci_longitudinal_s, &p);
    cd_longitudinal = strtof(cd_longitudinal_s, &p);

    cp_transversal_lr = strtof(cp_transversal_lr_s, &p);
    ci_transversal_lr = strtof(ci_transversal_lr_s, &p);
    cd_transversal_lr = strtof(cd_transversal_lr_s, &p);

    cp_transversal_diag = strtof(cp_transversal_diag_s, &p);
    ci_transversal_diag = strtof(ci_transversal_diag_s, &p);
    cd_transversal_diag = strtof(cd_transversal_diag_s, &p);

    cp_angle = strtof(cp_angle_s, &p);
}




void strreplace(char s[], char chr, char repl_chr)
{
     int i=0;
     while(s[i]!='\0')
     {
           if(s[i]==chr)
           {
               s[i]=repl_chr;
           }  
           i++; 
     }
          //printf("%s",s);
}






// ---- Machine Learning stuff for angle estimation ---- //
struct Scoeff_matrices read_ML_coeff(){
  //debug_print("func read_ML_coeff\n");
  struct Scoeff_matrices res;
  //if(kilo_uid != 2) {
    FILE *fPointer;
    char str[100];
    sprintf(str, "src/examples/behaviors/data/ML_coeff%i.csv", kilo_uid);
    fPointer = fopen(str, "r");

    

    //Step1
    char offset1_s[200*MULTIPLIER_INPUT];
    char gain1_s[200*MULTIPLIER_INPUT];
    char min1_s[200*MULTIPLIER_INPUT];

    double offset1[M1_DIM2];
    double gain1[M1_DIM2];
    double min1[M1_DIM2];

    fscanf(fPointer, "%s", offset1_s);
    fscanf(fPointer, "%s", gain1_s);
    fscanf(fPointer, "%s", min1_s);

    char *p;
    
    //Reading
    p = offset1_s;
    strreplace(offset1_s, ',', ' ');
    for (int i = 0 ; i < M1_DIM2 ; i++) {
        offset1[i] = strtof(p, &p);
    }

    p = gain1_s;
    strreplace(gain1_s, ',', ' ');
    for (int i = 0 ; i < M1_DIM2 ; i++) {
        gain1[i] = strtof(p, &p);
    }

    p = min1_s;
    strreplace(min1_s, ',', ' ');
    for (int i = 0 ; i < M1_DIM2 ; i++) {
        min1[i] = strtof(p, &p);
    }
  
    //Saving
    for (int i = 0; i < M1_DIM2; ++i)
    {
      res.offset1[i] = offset1[i];
      res.gain1[i] = gain1[i];
      res.min1[i] = min1[i];
    }

    //Step2
    char gain2_s[200*MULTIPLIER_INPUT];
    char bias22_s[200*MULTIPLIER_NEURON];

    double gain2[M1_DIM1][M1_DIM2];
    double bias22[M1_DIM1];


    //Reading
    for (int i = 0; i < M1_DIM1; ++i)
    {
      fscanf(fPointer, "%s", gain2_s);
      p = gain2_s;
      strreplace(gain2_s, ',', ' ');
      for (int j = 0 ; j < M1_DIM2 ; j++) {
          gain2[i][j] = strtof(p, &p);
      }
    }

    //Also reading
    fscanf(fPointer, "%s", bias22_s);
    p = bias22_s;
    strreplace(bias22_s, ',', ' ');
    for (int i = 0 ; i < M1_DIM1 ; i++) {
        bias22[i] = strtof(p, &p);
    }

    //Saving
    for (int i = 0; i < M1_DIM1; ++i)
    {
      res.bias22[i] = bias22[i];
      for (int j = 0; j < M1_DIM2; ++j)
      {
        res.gain2[i][j] = gain2[i][j];
      }
    }

    //Step3
    char gain3_s[200*MULTIPLIER_NEURON];
    char bias3_s[20];

    double gain3[M1_DIM1];
    double bias3;

    //Reading
    fscanf(fPointer, "%s", gain3_s);
    p = gain3_s;
    strreplace(gain3_s, ',', ' ');
    for (int i = 0 ; i < M1_DIM1 ; i++) {
        gain3[i] = strtof(p, &p);
    }

    fscanf(fPointer, "%s", bias3_s);
    sscanf(bias3_s, "%lf", &bias3);

    //Saving
    for (int i = 0; i < M1_DIM1; ++i)
    {
      res.gain3[i] = gain3[i];
    }
    res.bias3 = bias3;
    

    //Step4
    char min4_s[20];
    char gain4_s[20];
    char offset4_s[20];

    double min4;
    double gain4;
    double offset4;
    
    fscanf(fPointer, "%s", min4_s);
    fscanf(fPointer, "%s", gain4_s);
    fscanf(fPointer, "%s", offset4_s);

    sscanf(min4_s, "%lf", &min4);
    sscanf(gain4_s, "%lf", &gain4);
    sscanf(offset4_s, "%lf", &offset4);


    res.min4 = min4;
    res.gain4 = gain4;
    res.offset4 = offset4;

  //--> Kilo2 miért legyen kizárva már nem azért?
  //}//end if(kil_uid !=2)
  //debug_print("END func read_ML_coeff\n");
  return res;
}

void print_Scoeff_matrices(struct Scoeff_matrices coeff_matrices) {
    
    struct Scoeff_matrices res = coeff_matrices;
    //Step1
    printf(" // ---- Step1 ---- //\n");
    printf("offset1\t\t gain1\t\t\t min1\n");
    for (int i = 0; i < M1_DIM2; ++i)
    {
      printf("%lf\t", res.offset1[i] );
      printf("%lf\t", res.gain1[i] );
      printf("%lf\n", res.min1[i] );
    }

    //Step2
    printf("\n // ---- Step2 ---- //\n");
    printf(" |  bias22\t|\t\t gain2");
    for (int i = 0; i < M1_DIM1; ++i)
    {
      //res.bias22[i] = bias22[i];
      printf("\n | %f |  ", res.bias22[i]);
      for (int j = 0; j < M1_DIM2; ++j)
      {
        //res.gain2[i][j] = gain2[i][j];
        printf("%f  ", res.gain2[i][j]);
      }
    }

    //Step3
    printf("\n // ---- Step3 ---- //\n");
    printf(" gain3  \t--> ");
    
    for (int i = 0; i < M1_DIM1 -1; ++i)
    {
      printf("%lf,", res.gain3[i]);
    }
    printf("%lf\n", res.gain3[M1_DIM1-1] );

    printf(" bias3  \t--> %lf\n", res.bias3 );
    //Step4
    printf("\n // ---- Step3 ---- //\n");
    printf(" min4   \t--> %lf\n", res.min4);
    printf(" gain4  \t--> %lf\n", res.gain4);
    printf(" offset4\t--> %lf\n", res.offset4);
}


void array_manipulate(double *res, char *command, double *to_manip, int to_manip_size, double *manip_with, int manip_with_size) {
  for (int i = 0; i < to_manip_size; i++) {
    if (!strcmp(command, "add")) {
      res[i] = to_manip[i] + manip_with[i]; 
    } 
    else if (!strcmp(command, "substract")) {
      res[i] = to_manip[i] - manip_with[i]; 
    } 
    else if (!strcmp(command, "multiply")) {
      res[i] = to_manip[i] * manip_with[i]; 
    } 
    else if (!strcmp(command, "reverse divide")) {
      if (to_manip[i] == 0) { //to protect from dividing by zero. Quick fix, there are better ways...
        res[i] = 999999;
      } else {
        res[i] = manip_with[i] / to_manip[i]; 
      }
    } 
    else if (!strcmp(command, "sigmoid")) {     //note, that we are not using manip_with!!
      res[i] = 2 / ( 1 + exp(-2*to_manip[i])) - 1; 
    } 
  }
  
} //end of array_manipulate


//void matrix_multiply(double *res_matrix[M1_DIM1][M2_DIM2], double *matrix1[M1_DIM1][M1_DIM2], int m1_dim1, int m1_dim2, double *matrix2[M2_DIM1][M2_DIM2], int m2_dim1, int m2_dim2){
void matrix_multiply(double (*res_matrix)[M2_DIM2], double (*matrix1)[M1_DIM2], int m1_dim1, int m1_dim2, double (*matrix2)[M2_DIM2], int m2_dim1, int m2_dim2) {
    //Actual  matrix multiplication
    for (int i = 0; i < m1_dim1; ++i)
    {
      
      for (int j = 0; j < m2_dim2; ++j)
      {   
        double sum = 0;
        for (int k = 0; k < m1_dim2; ++k)
        {
          sum = sum + matrix1[i][k] * matrix2[k][j];
        }
        res_matrix[i][j] = sum;
      }
      
    }
    
}

void print_matrix(double (*matrix)[M1_DIM2], int m1_dim1, int m1_dim2) {

  for (int i = 0; i < m1_dim1; ++i)
  {
    for (int j = 0; j < m1_dim2; ++j)
    {
      printf("%f, \t", matrix[i][j] );
    }
    printf("\n");
  }
  printf("\n");

}

struct Seffectivenes_data get_effectivenes(int id_number){
    double time = 0;
    double one = 0;
    double two = 0;
    double three = 0;
    double four = 0;
    double five = 0;
    double six = 0;
    double seven = 0;
    double eight = 0;

  //if(id_number != 2) {
    FILE *fd;                           // File pointer
    char filename[100];
    sprintf(filename,"src/examples/behaviors/data/data%i.csv", id_number);       // file to read
    static const long max_len = 130+ 1;  // define the max length of the line to read
    char buff[max_len + 1];             // define the buffer and allocate the length
    
    

    if ((fd = fopen(filename, "r")) != NULL)  {      // open file. I omit error checks
        
        fseek(fd, -max_len, SEEK_END);            // set pointer to the end of file minus the length you need. Presumably there can be more than one new line caracter
        fread(buff, max_len-1, 1, fd);            // read the contents of the file starting from where fseek() positioned us
        fclose(fd);                               // close the file
        
        buff[max_len-1] = '\0';                   // close the string
        char *last_newline = strrchr(buff, '\n'); // find last occurrence of newlinw 
        char *last_line = last_newline+1;         // jump to it
        
        sscanf(last_line, "%lf, %lf, %lf, %lf, %lf, %lf, %lf, %lf, %lf\n", &time, &one, &two, &three, &four, &five, &six, &seven, &eight);
        
    }
  //--> sztem hagjuk őt is érvényesülni
  //}// end if kilo_uid != 2 
    struct Seffectivenes_data res;
    res.one = (double)one;
    res.two = (double)two;
    res.three = (double)three;
    res.four = (double)four;
    res.five = (double)five;
    res.six = (double)six;
    res.seven = (double)seven;
    res.eight = (double)eight;
  

    return res;
            
}

// MACHINE LEARNING CORE
double angle_calculation(double x1[M1_DIM2]){

// ---- Step 1---- //
  //Step 1.1
  int to_manip_size = M1_DIM2;
  int manip_with_size = M1_DIM2;


  //The command
  char command11[] = "substract";
  double y11[to_manip_size];

  //The function
  array_manipulate( y11, command11, x1, to_manip_size, coeff_matrices.offset1, manip_with_size);
  //y11 = 

  //The result
//  printf("x1 - offset1 = y11\n");
//  for (int i = 0; i < M1_DIM2; ++i)
//  {
//    printf("%lf\t", x1[i] );
//    printf("%lf\t", coeff_matrices.offset1[i] );
//    printf("%lf\n", y11[i] );
//  }
  
  //Step 1.2
  //double gain1[] = {2, 3, 4, 5, 6, 7};
  char command12[] = "multiply";
  double y12[to_manip_size];
  array_manipulate( y12, command12, y11, to_manip_size, coeff_matrices.gain1, manip_with_size);
  //y12 = 


  //Step 1.3
  //double min1[] = {2, 3, 4, 5, 6, 7};
  char command13[] = "add";
  double y13[to_manip_size];
  array_manipulate( y13, command13, y12, to_manip_size, coeff_matrices.min1, manip_with_size);

  //The result
//  printf("y12 - min1 = y13\n");
//  for (int i = 0; i < M1_DIM2; ++i)
//  {
//    printf("%lf\t", y12[i] );
//    printf("%lf\t", coeff_matrices.min1[i] );
//    printf("%lf\n", y13[i] );
//  }


// ---- Step 2 ---- //
  //Step 2.1
  const int m1_dim1 = M1_DIM1;
  const int m1_dim2 = M1_DIM2;
  

  const int m2_dim1 = M2_DIM1;
  const int m2_dim2 = M2_DIM2;


  double y13_transpose[M2_DIM1][M2_DIM2];

  //Just TRANSPOSE --> y13 is 1x6 
  //               --> y13_transpose is 6x1
  for (int i = 0; i < M1_DIM2; ++i)
  {
    y13_transpose[i][M2_DIM2-1] = y13[i];
  }


  double res_matrix[M1_DIM1][M2_DIM2];
  matrix_multiply(res_matrix, coeff_matrices.gain2, m1_dim1, m1_dim2, y13_transpose, m2_dim1, m2_dim2);
  
//  printf("After matrix multiplicationl\n");
//  for (int i = 0; i < M1_DIM1; ++i)
//  {
//    for (int j = 0; j < M2_DIM2; ++j)
//    {
//      printf("%lf\t", res_matrix[i][j]);
//    }
//    printf("\n");
//  }

  //Step 2.2
  double res_matrix_transpose[M1_DIM1];
  //Just TRANSPOSE --> res_matrix is 10x1 
  //               --> res_matrix_transpose is 1x10
  for (int i = 0; i < M1_DIM1; ++i)
  {
    res_matrix_transpose[i] = res_matrix[i][0];
  }

  char command22[] = "add";
  to_manip_size = (int)sizeof(res_matrix_transpose) / sizeof(double);
  manip_with_size = (int)sizeof(coeff_matrices.bias22) / sizeof(double);
  double y22[to_manip_size];
  array_manipulate( y22, command22, res_matrix_transpose, to_manip_size, coeff_matrices.bias22, manip_with_size);

//  printf("bias + res_matrix_transpose = y22\n");
//  for (int i = 0; i < to_manip_size; ++i)
//  {
//    printf("%lf \t %lf \t %lf\n", coeff_matrices.bias22[i], res_matrix_transpose[i], y22[i] );
//  }

  //Step 2.3
  char command23[] = "sigmoid";
  to_manip_size = (int)sizeof(y22) / sizeof(double);
  double y23[to_manip_size];
  array_manipulate( y23, command23, y22, to_manip_size, coeff_matrices.bias22, manip_with_size);

//  //printing out
//  printf("bias22 |\t y22 - > \t sigmoid y23\n\n");
//  for (int i = 0; i < to_manip_size; ++i)
//  {
//    printf("%lf\t %lf\t %lf\n", coeff_matrices.bias22[i], y22[i], y23[i] );
//  }
//  printf("\n");

  // ---- Step 3 ---- //
  double a = 0.0;
  for (int i = 0; i < M1_DIM1; ++i)
  {
    a = a + coeff_matrices.gain3[i] * y23[i];
  }
  a = a + coeff_matrices.bias3;
//  printf("%lf\n",a );
  
  // ---- Step 4 ---- //
  double angle = (a-coeff_matrices.min4) / coeff_matrices.gain4 + coeff_matrices.offset4;

//  printf("a - min4 , rdivide gain4 - offset4 = angle\n");
//  printf("%lf \t %lf \t %lf \t %lf \t %lf\n", a, coeff_matrices.min4, coeff_matrices.gain4, coeff_matrices.offset4, angle );

  return angle;
}















void write_data_MATLAB(double current_time) {

    struct SGPS_data GPS_data;
    GPS_data = get_location(kilo_uid);

    GPS_data_2 = get_location(2);
    GPS_data_4 = get_location(4);
    GPS_data_6 = get_location(6);

    FILE *fPointer_GOD;
    char str_GOD[100];
    sprintf(str_GOD, "src/examples/behaviors/data/data_GOD%i.csv", kilo_uid);
    fPointer_GOD = fopen(str_GOD, "a");
    fprintf(fPointer_GOD, "%f ,%f, %f, %f, %f, %f\n", 
      current_time, GPS_data.x, GPS_data.y, GPS_data.z, GPS_data.rotZ, true_angle);
    fclose(fPointer_GOD);

}

struct SGPS_data get_location(int id_number){
    FILE *fd;                           // File pointer
    char filename[100];
    sprintf(filename,"src/examples/loop_functions/trajectory_loop_functions/data/data%i.csv", id_number);       // file to read
    static const long max_len = 55+ 1;  // define the max length of the line to read
    char buff[max_len + 1];             // define the buffer and allocate the length
    double x = 0;
    double y = 0;
    double z = 0;
    double rotZ = 0;

    if ((fd = fopen(filename, "r")) != NULL)  {      // open file. I omit error checks
    
        fseek(fd, -max_len, SEEK_END);            // set pointer to the end of file minus the length you need. Presumably there can be more than one new line caracter
        fread(buff, max_len-1, 1, fd);            // read the contents of the file starting from where fseek() positioned us
        fclose(fd);                               // close the file
    
        buff[max_len-1] = '\0';                   // close the string
        char *last_newline = strrchr(buff, '\n'); // find last occurrence of newlinw 
        char *last_line = last_newline+1;         // jump to it
        
        sscanf(last_line, "%lf, %lf, %lf, %lf\n", &x, &y, &z, &rotZ);

    }

    struct SGPS_data res;
    res.x = (double)x;
    res.y = (double)y;
    res.z = (double)z;
    res.rotZ = (double)rotZ;
    return res;
            
}



void calculate_effectiveness(double current_time){

  delay_pid_longitudinal_old = delay_pid_longitudinal;
  delay_pid_transversal_old = delay_pid_transversal;

  double one = diversion_derivative[FRONT][NEW];
  double two = diversion_derivative[LEFT][NEW];
  double three = diversion_derivative[RIGHT][NEW];
  
  double four = diversion_derivative[FRONT][NEW];
  double five = diversion_derivative[LEFT][NEW];
  double six = diversion_derivative[RIGHT][NEW];

  double seven = delay_pid_longitudinal;
  double eight = delay_pid_transversal;

  effectivenes_data.one = (double)one;
  effectivenes_data.two = (double)two;
  effectivenes_data.three = (double)three;
  effectivenes_data.four = (double)four;
  effectivenes_data.five = (double)five;
  effectivenes_data.six = (double)six;
  effectivenes_data.seven = (double)seven - 70.0;
  effectivenes_data.eight = (double)eight;




  // Mapping effectivenes data to 0 - 255
  double one_max = 0.02;
  double one_min = -0.02;

  double two_max = 0.04;
  double two_min = -0.04;

  double three_max = two_max;
  double three_min = two_min;

  double four_max = one_max;
  double four_min = one_min;

  double five_max = two_max;
  double five_min = two_min;

  double six_max = two_max;
  double six_min = two_min;

  double seven_max = 30.0 / 1;
  double seven_min = -30.0 / 1;

  double eight_max = 127.0 / 1;
  double eight_min = -127.0 / 1;

  double mapping_max = 127.0;
  double mapping_min = -127.0;

  if (kilo_uid == 2)
  {
      two_max = 0.01; //vagy 0.002
      two_min = -0.01;
    
      three_max = 0.01;
      three_min = -0.01;
    
      five_max = two_max;
      five_min = two_min;
    
      six_max = three_max;
      six_min = three_min;
  }


  

  // Kilobot  //        
  effectivenes_data.one = effectivenes_data.one > one_max ? one_max : effectivenes_data.one;
  effectivenes_data.one = effectivenes_data.one < one_min ? one_min : effectivenes_data.one;
  effectivenes_data.one = effectivenes_data.one > 0 ? (effectivenes_data.one / one_max * mapping_max) : (effectivenes_data.one / one_min * mapping_min);

  effectivenes_data.two = effectivenes_data.two > two_max ? two_max : effectivenes_data.two;
  effectivenes_data.two = effectivenes_data.two < two_min ? two_min : effectivenes_data.two;
  effectivenes_data.two = effectivenes_data.two > 0 ? effectivenes_data.two / two_max * mapping_max : effectivenes_data.two / two_min * mapping_min;

  effectivenes_data.three = effectivenes_data.three > three_max ? three_max : effectivenes_data.three;
  effectivenes_data.three = effectivenes_data.three < three_min ? three_min : effectivenes_data.three;
  effectivenes_data.three = effectivenes_data.three > 0 ? effectivenes_data.three / three_max * mapping_max : effectivenes_data.three / three_min * mapping_min;

  effectivenes_data.four = effectivenes_data.four > four_max ? four_max : effectivenes_data.four;
  effectivenes_data.four = effectivenes_data.four < four_min ? four_min : effectivenes_data.four;
  effectivenes_data.four = effectivenes_data.four > 0 ? effectivenes_data.four / four_max * mapping_max : effectivenes_data.four / four_min * mapping_min;

  effectivenes_data.five = effectivenes_data.five > five_max ? five_max : effectivenes_data.five;
  effectivenes_data.five = effectivenes_data.five < five_min ? five_min : effectivenes_data.five;
  effectivenes_data.five = effectivenes_data.five > 0 ? effectivenes_data.five / five_max * mapping_max : effectivenes_data.five / five_min * mapping_min;

  effectivenes_data.six = effectivenes_data.six > six_max ? six_max : effectivenes_data.six;
  effectivenes_data.six = effectivenes_data.six < six_min ? six_min : effectivenes_data.six;
  effectivenes_data.six = effectivenes_data.six > 0 ? effectivenes_data.six / six_max * mapping_max : effectivenes_data.six / six_min * mapping_min;

  effectivenes_data.seven = effectivenes_data.seven > seven_max ? seven_max : effectivenes_data.seven;
  effectivenes_data.seven = effectivenes_data.seven < seven_min ? seven_min : effectivenes_data.seven;
  effectivenes_data.seven = effectivenes_data.seven > 0 ? effectivenes_data.seven / seven_max * mapping_max : effectivenes_data.seven / seven_min * mapping_min;

  effectivenes_data.eight = effectivenes_data.eight > eight_max ? eight_max : effectivenes_data.eight;
  effectivenes_data.eight = effectivenes_data.eight < eight_min ? eight_min : effectivenes_data.eight;
  effectivenes_data.eight = effectivenes_data.eight > 0 ? effectivenes_data.eight / eight_max * mapping_max : effectivenes_data.eight / eight_min * mapping_min;

  effectivenes_data.one    = round(effectivenes_data.one) + 127;
  effectivenes_data.two    = round(effectivenes_data.two) + 127;
  effectivenes_data.three  = round(effectivenes_data.three) + 127;
  effectivenes_data.four   = round(effectivenes_data.four) + 127;
  effectivenes_data.five   = round(effectivenes_data.five) + 127;
  effectivenes_data.six    = round(effectivenes_data.six) + 127;
  effectivenes_data.seven  = round(effectivenes_data.seven) + 127;
  effectivenes_data.eight  = round(effectivenes_data.eight) + 127;


  //To make things simmetric & make learning easier
  if(kilo_uid == 2 && effectivenes_data.eight == 127) {
    effectivenes_data.eight = 128;
  } 

//  if(kilo_uid == 2 && effectivenes_data.eight == 127) {
//    effectivenes_data.eight = -127;
//  } else if(kilo_uid == 2 && effectivenes_data.eight == 130) {
//    effectivenes_data.eight = 127;
//  } else if(kilo_uid == 2 && effectivenes_data.eight == 129 || effectivenes_data.eight == 0 ) {
//    effectivenes_data.eight = 0;
//  }



  switch(kilo_uid){
    case 2 :
      effectivenes_data2 = effectivenes_data;
      break;
    case 4 :
      effectivenes_data4 = effectivenes_data;
      break;
    case 6 :
      effectivenes_data6 = effectivenes_data;
      break;
  }
    
  //Updating message
  message.type = NORMAL;
  message.data[0] = kilo_uid;
  message.data[1] = effectivenes_data.one;
  message.data[2] = effectivenes_data.two;
  message.data[3] = effectivenes_data.three;
  message.data[4] = effectivenes_data.four;
  message.data[5] = effectivenes_data.five;
  message.data[6] = effectivenes_data.six;
  message.data[7] = effectivenes_data.seven;
  message.data[8] = effectivenes_data.eight;
  message.crc = message_crc(&message);

  //Waiting for synching: WAIT FOR MESSAGE UPDATE
  //32 ticks per second. Approximately 470ms per 15 ticks. We wait about 340ms after moving to synch up our messages.
  double sync_delay = WAIT_TIME - (kilo_ticks - prev_time);
  sync_delay = sync_delay / 32.0;
  sync_delay = sync_delay * 1000.0;
  delay( sync_delay );
  //debug_print("I waited %f ms inside calculate_effectiveness()\n", sync_delay);
  prev_time = kilo_ticks;
//  if (kilo_uid == 4)
//  {
//    printf("I waited %f ms\n", sync_delay);
//  }

  // Mapping effectivenes data to 0 - 255 END

  //Write data
  //Attention: in the setup we clear this file!
    
    FILE *fPointer;
    char str[100];
    sprintf(str, "src/examples/behaviors/data/data%i.csv", kilo_uid);
    fPointer = fopen(str, "a");
    fprintf(fPointer, "%f,%f, %f, %f, %f, %f, %f, %f, %f\n", current_time, one, two, three, four, five, six, seven, eight );
    fclose(fPointer);



    //Saving everything we have got:
    //Attention! We have to zero out these at the beginning!
    FILE *fPointer2;
    char str2[100];
    sprintf(str2, "src/examples/behaviors/data/data%i_2.csv", kilo_uid);
    fPointer2 = fopen(str2, "a");
    fprintf(fPointer2, "%f,%f, %f, %f, %f, %f, %f, %f, %f\n", current_time,
    effectivenes_data2.one, effectivenes_data2.two, effectivenes_data2.three, effectivenes_data2.four, 
    effectivenes_data2.five, effectivenes_data2.six, effectivenes_data2.seven, effectivenes_data2.eight );
    fclose(fPointer2);



    FILE *fPointer4;
    char str4[100];
    sprintf(str4, "src/examples/behaviors/data/data%i_4.csv", kilo_uid);
    fPointer4 = fopen(str4, "a");
    fprintf(fPointer4, "%f,%f, %f, %f, %f, %f, %f, %f, %f\n", current_time,
    effectivenes_data4.one, effectivenes_data4.two, effectivenes_data4.three, effectivenes_data4.four, 
    effectivenes_data4.five, effectivenes_data4.six, effectivenes_data4.seven, effectivenes_data4.eight );
    fclose(fPointer4);


    FILE *fPointer6;
    char str6[100];
    sprintf(str6, "src/examples/behaviors/data/data%i_6.csv", kilo_uid);
    fPointer6 = fopen(str6, "a");
    fprintf(fPointer6, "%f,%f, %f, %f, %f, %f, %f, %f, %f\n", current_time,
    effectivenes_data6.one, effectivenes_data6.two, effectivenes_data6.three, effectivenes_data6.four, 
    effectivenes_data6.five, effectivenes_data6.six, effectivenes_data6.seven, effectivenes_data6.eight );
    fclose(fPointer6);


    
}




//------ PID FUNCTIONS ------//
void calculate_pid_delay_longitudinal() {

          //P                                               //here simply an exponential is not good, because we are 'always behind' anyway. Maybe if we substract a given number, then... 
          proportional_part_longitudinal = cp_longitudinal * (diversion           [FRONT][NEW] - diversion           [BACK][NEW]);
          proportional_part_longitudinal = proportional_part_longitudinal < -1000 ? -1000 : proportional_part_longitudinal;  //Limiting integral
          proportional_part_longitudinal = proportional_part_longitudinal >  1000 ?  1000 : proportional_part_longitudinal;  //Limiting integral
          //I
          integral_part_longitudinal     = ci_longitudinal * (diversion_integral  [FRONT][NEW] - diversion_integral  [BACK][NEW]);
          //D
          derivative_part_longitudinal   = cd_longitudinal * (diversion_derivative[FRONT][NEW] - diversion_derivative[BACK][NEW]);
          //Sum
          delay_pid_longitudinal = proportional_part_longitudinal + integral_part_longitudinal + derivative_part_longitudinal;
         
}

void calculate_pid_delay_transversal() {

          //------ Proportional Part ------//
          double x, one, two, three, four, five, six;

          x = diversion[FRONTLEFT][NEW];
          one = ((x > 0) - (x < 0)) * pow(x,2);            //More serious, quadratic function
          
          x = diversion[LEFT][NEW];
          two = ((x > 0) - (x < 0)) * pow(x,2);

          x = diversion[BACKLEFT][NEW];
          three = x;

          x = diversion[FRONTRIGHT][NEW];
          four = ((x > 0) - (x < 0)) * pow(x,2);           //More serious, quadratic function
          
          x = diversion[RIGHT][NEW];
          five = ((x > 0) - (x < 0)) * pow(x,2);

          x = diversion[BACKRIGHT][NEW];
          six = x;

//          x = diversion[FRONTLEFT][NEW];
//          one = x;            //More serious, quadratic function
//          
//          x = diversion[LEFT][NEW];
//          two = x;
//
//          x = diversion[BACKLEFT][NEW];
//          three = x;
//
//          x = diversion[FRONTRIGHT][NEW];
//          four = x;           //More serious, quadratic function
//          
//          x = diversion[RIGHT][NEW];
//          five = x;
//
//          x = diversion[BACKRIGHT][NEW];
//          six = x;

          x = diversion[BACKRIGHT][NEW];
          six = x;

          proportional_part_transversal = cp_transversal_diag * one + cp_transversal_lr * two + cp_transversal_diag * three 
                                        - cp_transversal_diag * four - cp_transversal_lr * five - cp_transversal_diag * six;

          //Settin limits
          proportional_part_transversal = proportional_part_transversal < -100 ? -100 : proportional_part_transversal;  //Limiting integral
          proportional_part_transversal = proportional_part_transversal >  100 ?  100 : proportional_part_transversal;  //Limiting integral
          
          

          //------ Integral Part ------// 
          
          x = diversion_integral[FRONTLEFT][NEW];
          one = x;

          x = diversion_integral[LEFT][NEW];
          two = x;

          x = diversion_integral[BACKLEFT][NEW];
          three = x;

          x = diversion_integral[FRONTRIGHT][NEW];
          four = x;
          
          x = diversion_integral[RIGHT][NEW];
          five = x;

          x = diversion_integral[BACKRIGHT][NEW];
          six = x;

          integral_part_transversal = ci_transversal_diag * one + ci_transversal_lr * two + ci_transversal_diag * three 
                                        - ci_transversal_diag * four - ci_transversal_lr * five - ci_transversal_diag * six;

          //Setting limits
          integral_part_transversal = integral_part_transversal < -100 ? -100 : integral_part_transversal;  //Limiting integral
          integral_part_transversal = integral_part_transversal >  100 ?  100 : integral_part_transversal;  //Limiting integral
          
          //------ Derivative Part ------// 

          x = diversion_derivative[FRONTLEFT][NEW];
          one = x;

          x = diversion_derivative[LEFT][NEW];
          two = x;

          x = diversion_derivative[BACKLEFT][NEW];
          three = x;

          x = diversion_derivative[FRONTRIGHT][NEW];
          four = x;
          
          x = diversion_derivative[RIGHT][NEW];
          five = x;

          x = diversion_derivative[BACKRIGHT][NEW];
          six = x;

          derivative_part_transversal = cd_transversal_diag * one + cd_transversal_lr * two + cd_transversal_diag * three 
                                        - cd_transversal_diag * four - cd_transversal_lr * five - cd_transversal_diag * six;

          //Setting limits
          derivative_part_transversal = derivative_part_transversal < -100 ? -100 : derivative_part_transversal; //Limiting derivative
          derivative_part_transversal = derivative_part_transversal >  100 ?  100 : derivative_part_transversal; //Limiting derivative
          
          //------ Sum ------// 
          delay_pid_transversal = proportional_part_transversal 
                                  + integral_part_transversal 
                                  + derivative_part_transversal;
          
          //Setting limits
          delay_pid_transversal = delay_pid_transversal >  100 ?  100 : delay_pid_transversal; //Limiting pid transversal
          

}


//------ Miscellaneous FUNCTIONS ------//
void set_motion(int new_motion) {
   // Only take an action if the motion is being changed.
   if (current_motion != new_motion)
   {
      current_motion = new_motion;
        
      if (current_motion == STOP)
      {
         set_motors(0, 0);
      }
      else if (current_motion == FORWARD)
      {
         spinup_motors();
         set_motors(kilo_straight_left, kilo_straight_right);
      }
      else if (current_motion == LEFT)
      {
         spinup_motors();
         set_motors(kilo_turn_left, 0);
      }
      else if (current_motion == RIGHT)
      {
         spinup_motors();
         set_motors(0, kilo_turn_right);
      }
   }
}


void message_rx(message_t *m, distance_measurement_t *d)
{
    new_message = 1;
    received_ID = m->data[0];
    distance = estimate_distance(d);

    //Stuff for ML
    effectivenes_data_received.one    = m->data[1];
    effectivenes_data_received.two    = m->data[2];
    effectivenes_data_received.three  = m->data[3];
    effectivenes_data_received.four   = m->data[4];
    effectivenes_data_received.five   = m->data[5];
    effectivenes_data_received.six    = m->data[6];
    effectivenes_data_received.seven  = m->data[7];
    effectivenes_data_received.eight  = m->data[8];

    switch(received_ID){
      case 2 :
        effectivenes_data2 = effectivenes_data_received;
        break;
      case 4 :
        effectivenes_data4 = effectivenes_data_received;
        break;
      case 6 :
        effectivenes_data6 = effectivenes_data_received;
        break;
    }

    //Stuff for ML end
    if(received_ID == neighbours[FRONT]){
        distances [FRONT][DISTANCE] = distance;
        distances [FRONT][TICK]     = kilo_ticks;
    }
    //Azért nincs else, mert egy Kilobot több szerepet is betölthet!!!!
    //Lehet egyszerre front neighbour and diagonal neighbour
     if(received_ID == neighbours[BACK]){
        distances [BACK][DISTANCE] = distance;
        distances [BACK][TICK]     = kilo_ticks;
    }
     if(received_ID == neighbours[LEFT]){
        distances [LEFT][DISTANCE] = distance;
        distances [LEFT][TICK]     = kilo_ticks;  
    }
     if(received_ID == neighbours[RIGHT]){
        distances [RIGHT][DISTANCE] = distance;
        distances [RIGHT][TICK]     = kilo_ticks;
    }
     if(received_ID == neighbours[FRONTLEFT]){
        distances [FRONTLEFT][DISTANCE] = distance;
        distances [FRONTLEFT][TICK]     = kilo_ticks;
    }
     if(received_ID == neighbours[FRONTRIGHT]){
        distances [FRONTRIGHT][DISTANCE] = distance;
        distances [FRONTRIGHT][TICK]     = kilo_ticks;
    }
     if(received_ID == neighbours[BACKLEFT]){
        distances [BACKLEFT][DISTANCE] = distance;
        distances [BACKLEFT][TICK]     = kilo_ticks;
    }
     if(received_ID == neighbours[BACKRIGHT]){
        distances [BACKRIGHT][DISTANCE] = distance;
        distances [BACKRIGHT][TICK]     = kilo_ticks;
    }

 }

 message_t *message_tx() {
    return &message;
}
